@@ -1,34 +1,36 @@
 /***********************************************************************
-* SPDX-FileCopyrightText: 2003-2004 Max Howell <max.howell@methylblue.com>
-* SPDX-FileCopyrightText: 2008-2009 Martin Sandsmark <martin.sandsmark@kde.org>
-*
-* SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
-***********************************************************************/
+ * SPDX-FileCopyrightText: 2003-2004 Max Howell <max.howell@methylblue.com>
+ * SPDX-FileCopyrightText: 2008-2009 Martin Sandsmark <martin.sandsmark@kde.org>
+ * SPDX-FileCopyrightText: 2022 Harald Sitter <sitter@kde.org>
+ *
+ * SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
+ ***********************************************************************/
 
 #include "scan.h"
 
-#include "remoteLister.h"
 #include "fileTree.h"
-#include "localLister.h"
 #include "filelight_debug.h"
+#include "localLister.h"
+#include "remoteLister.h"
 
-#include <QGuiApplication>
 #include <QCursor>
 #include <QDir>
+#include <QGuiApplication>
 #include <QStringBuilder>
 
 namespace Filelight
 {
 
 ScanManager::ScanManager(QObject *parent)
-        : QObject(parent)
-        , m_abort(false)
-        , m_files(0)
-        , m_mutex()
-        , m_thread(nullptr)
+    : QObject(parent)
+    , m_abort(false)
+    , m_files(0)
+    , m_thread(nullptr)
 {
     Filelight::LocalLister::readMounts();
     connect(this, &ScanManager::branchCacheHit, this, &ScanManager::foundCached, Qt::QueuedConnection);
+    // Bit aggressive this. Completed is emitted per folder I think.
+    connect(this, &ScanManager::completed, this, &ScanManager::runningChanged, Qt::QueuedConnection);
 }
 
 ScanManager::~ScanManager()
@@ -40,19 +42,19 @@ ScanManager::~ScanManager()
     }
     qDeleteAll(m_cache);
 
-    //RemoteListers are QObjects and get automatically deleted
+    // RemoteListers are QObjects and get automatically deleted
 }
 
 bool ScanManager::running() const
 {
-    return m_thread && m_thread->isRunning();
+    return (m_thread && m_thread->isRunning()) || (m_remoteLister && !m_remoteLister->isFinished());
 }
 
 bool ScanManager::start(const QUrl &url)
 {
     QMutexLocker locker(&m_mutex); // The m_mutex gets released once locker is destroyed (goes out of scope).
 
-    //url is guaranteed clean and safe
+    // url is guaranteed clean and safe
 
     qCDebug(FILELIGHT_LOG) << "Scan requested for: " << url;
 
@@ -67,12 +69,19 @@ bool ScanManager::start(const QUrl &url)
 
     if (!url.isLocalFile()) {
         QGuiApplication::changeOverrideCursor(Qt::BusyCursor);
-        //will start listing straight away
-        Filelight::RemoteLister *remoteLister = new Filelight::RemoteLister(url, (QWidget*)parent(), this);
-        connect(remoteLister, &Filelight::RemoteLister::branchCompleted, this, &ScanManager::cacheTree, Qt::QueuedConnection);
-        remoteLister->setParent(this);
-        remoteLister->setObjectName(QStringLiteral( "remote_lister" ));
-        remoteLister->openUrl(url);
+        // will start listing straight away
+        m_remoteLister = std::make_unique<Filelight::RemoteLister>(url, this);
+        connect(m_remoteLister.get(), &Filelight::RemoteLister::branchCompleted, this, &ScanManager::cacheTree, Qt::QueuedConnection);
+        auto updateRunning = [this] {
+            if (m_remoteLister && m_remoteLister->isFinished()) {
+                m_remoteLister = nullptr;
+                Q_EMIT runningChanged();
+            }
+        };
+        connect(m_remoteLister.get(), &Filelight::RemoteLister::completed, this, updateRunning);
+        connect(m_remoteLister.get(), &Filelight::RemoteLister::canceled, this, updateRunning);
+        m_remoteLister->openUrl(url);
+        Q_EMIT runningChanged();
         return true;
     }
 
@@ -87,41 +96,44 @@ bool ScanManager::start(const QUrl &url)
         path += QLatin1Char('/');
     }
 
-    QList<Folder*> *trees = new QList<Folder*>;
+    auto *trees = new QList<Folder *>;
 
     /* CHECK CACHE
-         *   user wants: /usr/local/
-         *   cached:     /usr/
-         *
-         *   user wants: /usr/
-         *   cached:     /usr/local/, /usr/include/
-         */
+     *   user wants: /usr/local/
+     *   cached:     /usr/
+     *
+     *   user wants: /usr/
+     *   cached:     /usr/local/, /usr/include/
+     */
 
-    QMutableListIterator<Folder*> it(m_cache);
+    QMutableListIterator<Folder *> it(m_cache);
     while (it.hasNext()) {
         Folder *folder = it.next();
         QString cachePath = folder->decodedName();
 
-        if (path.startsWith(cachePath)) { //then whole tree already scanned
-            //find a pointer to the requested branch
+        if (path.startsWith(cachePath)) { // then whole tree already scanned
+            // find a pointer to the requested branch
 
             qCDebug(FILELIGHT_LOG) << "Cache-(a)hit: " << cachePath;
-
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
             QVector<QStringRef> split = path.midRef(cachePath.length()).split(QLatin1Char('/'));
+#else
+            QVector<QStringView> split = QStringView(path).mid(cachePath.length()).split(QLatin1Char('/'));
+#endif
             Folder *d = folder;
 
-            while (!split.isEmpty() && d != nullptr) { //if NULL we have got lost so abort!!
-                if (split.first().isEmpty()) { //found the dir
+            while (!split.isEmpty() && d != nullptr) { // if NULL we have got lost so abort!!
+                if (split.first().isEmpty()) { // found the dir
                     break;
                 }
                 QString s = split.first() % QLatin1Char('/'); // % is the string concatenation operator for QStringBuilder
 
-                QListIterator<File*> it(d->files);
+                QListIterator<File *> it(d->files);
                 d = nullptr;
                 while (it.hasNext()) {
                     File *subfolder = it.next();
                     if (s == subfolder->decodedName()) {
-                        d = (Folder*)subfolder;
+                        d = (Folder *)subfolder;
                         break;
                     }
                 }
@@ -132,21 +144,20 @@ bool ScanManager::start(const QUrl &url)
             if (d) {
                 delete trees;
 
-                //we found a completed tree, thus no need to scan
+                // we found a completed tree, thus no need to scan
                 qCDebug(FILELIGHT_LOG) << "Found cache-handle, generating map..";
 
                 Q_EMIT branchCacheHit(d);
 
                 return true;
-            } else {
-                //something went wrong, we couldn't find the folder we were expecting
-                qCWarning(FILELIGHT_LOG) << "Didn't find " << path << " in the cache!\n";
-                it.remove();
-                Q_EMIT aboutToEmptyCache();
-                delete folder;
-                break; //do a full scan
-            }
-        }  else if (cachePath.startsWith(path)) { //then part of the requested tree is already scanned
+            } // something went wrong, we couldn't find the folder we were expecting
+            qCWarning(FILELIGHT_LOG) << "Didn't find " << path << " in the cache!\n";
+            it.remove();
+            Q_EMIT aboutToEmptyCache();
+            delete folder;
+            break; // do a full scan
+        }
+        if (cachePath.startsWith(path)) { // then part of the requested tree is already scanned
             qCDebug(FILELIGHT_LOG) << "Cache-(b)hit: " << cachePath;
             it.remove();
             trees->append(folder);
@@ -154,11 +165,13 @@ bool ScanManager::start(const QUrl &url)
     }
 
     QGuiApplication::changeOverrideCursor(QCursor(Qt::BusyCursor));
-    //starts listing by itself
+    // starts listing by itself
     m_thread = new Filelight::LocalLister(path, trees, this);
     connect(m_thread, &LocalLister::branchCompleted, this, &ScanManager::cacheTree, Qt::QueuedConnection);
     m_thread->start();
 
+    Q_EMIT runningChanged();
+
     return true;
 }
 
@@ -166,9 +179,15 @@ bool ScanManager::abort()
 {
     m_abort = true;
 
-    delete findChild<RemoteLister *>(QStringLiteral( "remote_lister" ));
+    if (m_remoteLister) {
+        m_remoteLister->stop();
+    }
+    m_remoteLister = nullptr;
+    const bool ret = m_thread && m_thread->wait();
+    Q_EMIT runningChanged();
 
-    return m_thread && m_thread->wait();
+    Q_EMIT aborted();
+    return ret;
 }
 
 void ScanManager::invalidateCacheFor(const QUrl &url)
@@ -192,9 +211,9 @@ void ScanManager::invalidateCacheFor(const QUrl &url)
 
     Q_EMIT aboutToEmptyCache();
 
-    QMutableListIterator<Folder*> cacheIterator(m_cache);
-    QList<Folder*> subCaches;
-    QList<Folder*> oldCache = m_cache;
+    QMutableListIterator<Folder *> cacheIterator(m_cache);
+    QList<Folder *> subCaches;
+    QList<Folder *> oldCache = m_cache;
     while (cacheIterator.hasNext()) {
         Folder *folder = cacheIterator.next();
         cacheIterator.remove();
@@ -204,17 +223,20 @@ void ScanManager::invalidateCacheFor(const QUrl &url)
         if (!path.startsWith(cachePath)) {
             continue;
         }
-
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
         QVector<QStringRef> splitPath = path.midRef(cachePath.length()).split(QLatin1Char('/'));
+#else
+        QVector<QStringView> splitPath = QStringView(path).mid(cachePath.length()).split(QLatin1Char('/'));
+#endif
         Folder *d = folder;
 
-        while (!splitPath.isEmpty() && d != nullptr) { //if NULL we have got lost so abort!!
-            if (splitPath.first().isEmpty()) { //found the dir
+        while (!splitPath.isEmpty() && d != nullptr) { // if NULL we have got lost so abort!!
+            if (splitPath.first().isEmpty()) { // found the dir
                 break;
             }
             QString wantedName = splitPath.takeFirst() % QLatin1Char('/'); // % is the string concatenation operator for QStringBuilder
 
-            QListIterator<File*> it(d->files);
+            QListIterator<File *> it(d->files);
             d = nullptr;
             while (it.hasNext()) {
                 File *subfolder = it.next();
@@ -226,7 +248,7 @@ void ScanManager::invalidateCacheFor(const QUrl &url)
                     continue;
                 }
 
-                Folder *newFolder = ((Folder*)subfolder)->duplicate();
+                Folder *newFolder = ((Folder *)subfolder)->duplicate();
                 newFolder->setName((cachePath.toLocal8Bit() + subfolder->name8Bit()));
                 subCaches.append(newFolder);
                 d = nullptr;
@@ -267,18 +289,18 @@ void ScanManager::cacheTree(Folder *tree)
         qCDebug(FILELIGHT_LOG) << "Waiting for thread to terminate ...";
         m_thread->wait();
         qCDebug(FILELIGHT_LOG) << "Thread terminated!";
-        delete m_thread; //note the lister deletes itself
+        delete m_thread; // note the lister deletes itself
         m_thread = nullptr;
     }
 
     Q_EMIT completed(tree);
 
     if (tree) {
-        //we don't cache foreign stuff
-        //we don't recache stuff (thus only type 1000 events)
-        //we always just have one tree cached, so we really don't need a list..
+        // we don't cache foreign stuff
+        // we don't recache stuff (thus only type 1000 events)
+        // we always just have one tree cached, so we really don't need a list..
         m_cache.append(tree);
-    } else { //scan failed
+    } else { // scan failed
         qDeleteAll(m_cache);
         m_cache.clear();
     }
@@ -292,7 +314,4 @@ void ScanManager::foundCached(Folder *tree)
     QGuiApplication::restoreOverrideCursor();
 }
 
-
 }
-
-
